@@ -9,7 +9,7 @@ import sys
 import threading
 from collections.abc import Generator
 from dataclasses import dataclass
-from io import TextIOWrapper
+from io import StringIO, TextIOWrapper
 from logging import Filter
 from typing import Any, ClassVar
 
@@ -29,7 +29,7 @@ from rich.style import Style
 from pip._internal.utils._log import VERBOSE, getLogger
 from pip._internal.utils.compat import WINDOWS
 from pip._internal.utils.deprecation import DEPRECATION_MSG_PREFIX
-from pip._internal.utils.misc import ensure_dir
+from pip._internal.utils.misc import StreamWrapper, ensure_dir
 
 _log_state = threading.local()
 _stdout_console = None
@@ -56,6 +56,38 @@ def _is_broken_pipe_error(exc_class: type[BaseException], exc: BaseException) ->
     return isinstance(exc, OSError) and exc.errno in (errno.EINVAL, errno.EPIPE)
 
 
+@contextlib.contextmanager
+def capture_logging() -> Generator[StringIO, None, None]:
+    """Capture all pip logs in a buffer temporarily."""
+    # Patching sys.std(out|err) directly is not viable as the caller
+    # may want to emit non-logging output (e.g. a rich spinner). To
+    # avoid capturing that, temporarily patch the root logging handlers
+    # to use new rich consoles that write to a StringIO.
+    handlers = {}
+    for handler in logging.getLogger().handlers:
+        if isinstance(handler, RichPipStreamHandler):
+            # Also store the handler's original console so it can be
+            # restored on context exit.
+            handlers[handler] = handler.console
+
+    fake_stream = StreamWrapper.from_stream(sys.stdout)
+    if not handlers:
+        yield fake_stream
+        return
+
+    # HACK: grab no_color attribute from a random handler console since
+    # it's a global option anyway.
+    no_color = next(iter(handlers.values())).no_color
+    fake_console = PipConsole(file=fake_stream, no_color=no_color, soft_wrap=True)
+    try:
+        for handler in handlers:
+            handler.console = fake_console
+        yield fake_stream
+    finally:
+        for handler, original_console in handlers.items():
+            handler.console = original_console
+
+
 @contextlib.contextmanager
 def indent_log(num: int = 2) -> Generator[None, None, None]:
     """
